/*     */ package net.rubyeye.xmemcached.utils.hibernate;
/*     */ 
/*     */ import com.google.code.yanf4j.config.Configuration;
/*     */ import com.googlecode.hibernate.memcached.Memcache;
/*     */ import com.googlecode.hibernate.memcached.MemcacheClientFactory;
/*     */ import com.googlecode.hibernate.memcached.PropertiesHelper;
/*     */ import net.rubyeye.xmemcached.CommandFactory;
/*     */ import net.rubyeye.xmemcached.HashAlgorithm;
/*     */ import net.rubyeye.xmemcached.MemcachedClient;
/*     */ import net.rubyeye.xmemcached.MemcachedClientBuilder;
/*     */ import net.rubyeye.xmemcached.MemcachedSessionLocator;
/*     */ import net.rubyeye.xmemcached.XMemcachedClientBuilder;
/*     */ import net.rubyeye.xmemcached.command.BinaryCommandFactory;
/*     */ import net.rubyeye.xmemcached.command.TextCommandFactory;
/*     */ import net.rubyeye.xmemcached.impl.ArrayMemcachedSessionLocator;
/*     */ import net.rubyeye.xmemcached.impl.KetamaMemcachedSessionLocator;
/*     */ import net.rubyeye.xmemcached.impl.PHPMemcacheSessionLocator;
/*     */ import net.rubyeye.xmemcached.utils.AddrUtil;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class XmemcachedClientFactory
/*     */   implements MemcacheClientFactory
/*     */ {
/*     */   public static final String PROP_SERVERS = "hibernate.memcached.servers";
/*     */   public static final String PROP_READ_BUFFER_SIZE = "hibernate.memcached.readBufferSize";
/*     */   public static final String PROP_OPERATION_TIMEOUT = "hibernate.memcached.operationTimeout";
/*     */   public static final String PROP_HASH_ALGORITHM = "hibernate.memcached.hashAlgorithm";
/*     */   public static final String PROP_COMMAND_FACTORY = "hibernate.memcached.commandFactory";
/*     */   public static final String PROP_SESSION_LOCATOR = "hibernate.memcached.sessionLocator";
/*     */   public static final String PROP_CONNECTION_POOL_SIZE = "hibernate.memcached.connectionPoolSize";
/*     */   public static final String PROP_CONNECT_TIMEOUT = "hibernate.memcached.connectTimeout";
/*     */   private final PropertiesHelper properties;
/*     */   
/*     */   public XmemcachedClientFactory(PropertiesHelper properties)
/*     */   {
/*  74 */     this.properties = properties;
/*     */   }
/*     */   
/*     */   public Memcache createMemcacheClient() throws Exception {
/*  78 */     MemcachedClientBuilder builder = new XMemcachedClientBuilder(AddrUtil.getAddresses(getServerList()));
/*     */     
/*  80 */     builder.setCommandFactory(getCommandFactory());
/*  81 */     builder.setSessionLocator(getSessionLocator());
/*  82 */     builder.getConfiguration().setSessionReadBufferSize(getReadBufferSize());
/*     */     
/*  84 */     builder.setConnectionPoolSize(getConnectionPoolSize());
/*  85 */     builder.setConnectTimeout(getConnectTimeoutMillis());
/*  86 */     MemcachedClient client = builder.build();
/*  87 */     client.setOpTimeout(getOperationTimeoutMillis());
/*  88 */     return new Xmemcache(client);
/*     */   }
/*     */   
/*     */   protected MemcachedSessionLocator getSessionLocator() {
/*  92 */     if (sessionLocatorNameEquals(ArrayMemcachedSessionLocator.class)) {
/*  93 */       return new ArrayMemcachedSessionLocator(getHashAlgorithm());
/*     */     }
/*     */     
/*  96 */     if (sessionLocatorNameEquals(KetamaMemcachedSessionLocator.class)) {
/*  97 */       return new KetamaMemcachedSessionLocator(getHashAlgorithm());
/*     */     }
/*     */     
/* 100 */     if (sessionLocatorNameEquals(PHPMemcacheSessionLocator.class)) {
/* 101 */       return new PHPMemcacheSessionLocator(getHashAlgorithm());
/*     */     }
/*     */     
/* 104 */     throw new IllegalArgumentException("Unsupported hibernate.memcached.sessionLocator value: " + getCommandFactoryName());
/*     */   }
/*     */   
/*     */   protected CommandFactory getCommandFactory()
/*     */   {
/* 109 */     if (commandFactoryNameEquals(TextCommandFactory.class)) {
/* 110 */       return new TextCommandFactory();
/*     */     }
/*     */     
/* 113 */     if (commandFactoryNameEquals(BinaryCommandFactory.class)) {
/* 114 */       return new BinaryCommandFactory();
/*     */     }
/*     */     
/* 117 */     throw new IllegalArgumentException("Unsupported hibernate.memcached.commandFactory value: " + getCommandFactoryName());
/*     */   }
/*     */   
/*     */   private boolean commandFactoryNameEquals(Class<?> cls)
/*     */   {
/* 122 */     return cls.getSimpleName().equals(getCommandFactoryName());
/*     */   }
/*     */   
/*     */   private boolean sessionLocatorNameEquals(Class<?> cls) {
/* 126 */     return cls.getSimpleName().equals(getSessionLocatorName());
/*     */   }
/*     */   
/*     */   public String getServerList() {
/* 130 */     return this.properties.get("hibernate.memcached.servers", "localhost:11211");
/*     */   }
/*     */   
/*     */   public int getReadBufferSize() {
/* 134 */     return this.properties.getInt("hibernate.memcached.readBufferSize", 131072);
/*     */   }
/*     */   
/*     */ 
/*     */   public int getConnectionPoolSize()
/*     */   {
/* 140 */     return this.properties.getInt("hibernate.memcached.connectionPoolSize", 1);
/*     */   }
/*     */   
/*     */   public long getOperationTimeoutMillis()
/*     */   {
/* 145 */     return this.properties.getLong("hibernate.memcached.operationTimeout", 5000L);
/*     */   }
/*     */   
/*     */   public long getConnectTimeoutMillis()
/*     */   {
/* 150 */     return this.properties.getLong("hibernate.memcached.connectTimeout", 60000L);
/*     */   }
/*     */   
/*     */   public HashAlgorithm getHashAlgorithm()
/*     */   {
/* 155 */     return (HashAlgorithm)this.properties.getEnum("hibernate.memcached.hashAlgorithm", HashAlgorithm.class, HashAlgorithm.NATIVE_HASH);
/*     */   }
/*     */   
/*     */   public String getCommandFactoryName()
/*     */   {
/* 160 */     return this.properties.get("hibernate.memcached.commandFactory", TextCommandFactory.class.getSimpleName());
/*     */   }
/*     */   
/*     */   public String getSessionLocatorName()
/*     */   {
/* 165 */     return this.properties.get("hibernate.memcached.sessionLocator", ArrayMemcachedSessionLocator.class.getSimpleName());
/*     */   }
/*     */   
/*     */   protected PropertiesHelper getProperties()
/*     */   {
/* 170 */     return this.properties;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\net\rubyeye\xmemcached\utils\hibernate\XmemcachedClientFactory.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */