/*     */ package org.apache.avro.file;
/*     */ 
/*     */ import java.io.EOFException;
/*     */ import java.io.File;
/*     */ import java.io.IOException;
/*     */ import java.io.InputStream;
/*     */ import java.util.Arrays;
/*     */ import org.apache.avro.io.BinaryDecoder;
/*     */ import org.apache.avro.io.DatumReader;
/*     */ import org.apache.avro.io.DecoderFactory;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class DataFileReader<D>
/*     */   extends DataFileStream<D>
/*     */   implements FileReader<D>
/*     */ {
/*     */   private SeekableInputStream sin;
/*     */   private long blockStart;
/*     */   
/*     */   public static <D> FileReader<D> openReader(File file, DatumReader<D> reader)
/*     */     throws IOException
/*     */   {
/*  42 */     return openReader(new SeekableFileInput(file), reader);
/*     */   }
/*     */   
/*     */ 
/*     */   public static <D> FileReader<D> openReader(SeekableInput in, DatumReader<D> reader)
/*     */     throws IOException
/*     */   {
/*  49 */     if (in.length() < DataFileConstants.MAGIC.length) {
/*  50 */       throw new IOException("Not an Avro data file");
/*     */     }
/*     */     
/*  53 */     byte[] magic = new byte[DataFileConstants.MAGIC.length];
/*  54 */     in.seek(0L);
/*  55 */     for (int c = 0; c < magic.length; c = in.read(magic, c, magic.length - c)) {}
/*  56 */     in.seek(0L);
/*     */     
/*  58 */     if (Arrays.equals(DataFileConstants.MAGIC, magic))
/*  59 */       return new DataFileReader(in, reader);
/*  60 */     if (Arrays.equals(DataFileReader12.MAGIC, magic)) {
/*  61 */       return new DataFileReader12(in, reader);
/*     */     }
/*  63 */     throw new IOException("Not an Avro data file");
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static <D> DataFileReader<D> openReader(SeekableInput in, DatumReader<D> reader, DataFileStream.Header header, boolean sync)
/*     */     throws IOException
/*     */   {
/*  78 */     DataFileReader<D> dreader = new DataFileReader(in, reader, header);
/*     */     
/*  80 */     if (sync) {
/*  81 */       dreader.sync(in.tell());
/*     */     } else
/*  83 */       dreader.seek(in.tell());
/*  84 */     return dreader;
/*     */   }
/*     */   
/*     */   public DataFileReader(File file, DatumReader<D> reader) throws IOException
/*     */   {
/*  89 */     this(new SeekableFileInput(file), reader);
/*     */   }
/*     */   
/*     */   public DataFileReader(SeekableInput sin, DatumReader<D> reader)
/*     */     throws IOException
/*     */   {
/*  95 */     super(reader);
/*  96 */     this.sin = new SeekableInputStream(sin);
/*  97 */     initialize(this.sin);
/*  98 */     blockFinished();
/*     */   }
/*     */   
/*     */ 
/*     */   protected DataFileReader(SeekableInput sin, DatumReader<D> reader, DataFileStream.Header header)
/*     */     throws IOException
/*     */   {
/* 105 */     super(reader);
/* 106 */     this.sin = new SeekableInputStream(sin);
/* 107 */     initialize(this.sin, header);
/*     */   }
/*     */   
/*     */ 
/*     */   public void seek(long position)
/*     */     throws IOException
/*     */   {
/* 114 */     this.sin.seek(position);
/* 115 */     this.vin = DecoderFactory.get().binaryDecoder(this.sin, this.vin);
/* 116 */     this.datumIn = null;
/* 117 */     this.blockRemaining = 0L;
/* 118 */     this.blockStart = position;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public void sync(long position)
/*     */     throws IOException
/*     */   {
/* 127 */     seek(position);
/*     */     
/* 129 */     if ((position == 0L) && (getMeta("avro.sync") != null)) {
/* 130 */       initialize(this.sin);
/* 131 */       return;
/*     */     }
/*     */     try {
/* 134 */       int i = 0;
/* 135 */       InputStream in = this.vin.inputStream();
/* 136 */       this.vin.readFixed(this.syncBuffer);
/*     */       int b;
/* 138 */       do { for (int j = 0; 
/* 139 */             j < 16; j++) {
/* 140 */           if (getHeader().sync[j] != this.syncBuffer[((i + j) % 16)])
/*     */             break;
/*     */         }
/* 143 */         if (j == 16) {
/* 144 */           this.blockStart = (position + i + 16L);
/* 145 */           return;
/*     */         }
/* 147 */         b = in.read();
/* 148 */         this.syncBuffer[(i++ % 16)] = ((byte)b);
/* 149 */       } while (b != -1);
/*     */     }
/*     */     catch (EOFException e) {}
/*     */     
/*     */ 
/* 154 */     this.blockStart = this.sin.tell();
/*     */   }
/*     */   
/*     */ 
/*     */   protected void blockFinished()
/*     */     throws IOException
/*     */   {
/* 161 */     this.blockStart = (this.sin.tell() - this.vin.inputStream().available());
/*     */   }
/*     */   
/*     */   public long previousSync()
/*     */   {
/* 166 */     return this.blockStart;
/*     */   }
/*     */   
/*     */   public boolean pastSync(long position)
/*     */     throws IOException
/*     */   {
/* 172 */     return (this.blockStart >= position + 16L) || (this.blockStart >= this.sin.length());
/*     */   }
/*     */   
/* 175 */   public long tell() throws IOException { return this.sin.tell(); }
/*     */   
/*     */   static class SeekableInputStream extends InputStream implements SeekableInput
/*     */   {
/* 179 */     private final byte[] oneByte = new byte[1];
/*     */     private SeekableInput in;
/*     */     
/*     */     SeekableInputStream(SeekableInput in) throws IOException {
/* 183 */       this.in = in;
/*     */     }
/*     */     
/*     */     public void seek(long p) throws IOException
/*     */     {
/* 188 */       if (p < 0L)
/* 189 */         throw new IOException("Illegal seek: " + p);
/* 190 */       this.in.seek(p);
/*     */     }
/*     */     
/*     */     public long tell() throws IOException
/*     */     {
/* 195 */       return this.in.tell();
/*     */     }
/*     */     
/*     */     public long length() throws IOException
/*     */     {
/* 200 */       return this.in.length();
/*     */     }
/*     */     
/*     */     public int read(byte[] b) throws IOException
/*     */     {
/* 205 */       return this.in.read(b, 0, b.length);
/*     */     }
/*     */     
/*     */     public int read(byte[] b, int off, int len) throws IOException
/*     */     {
/* 210 */       return this.in.read(b, off, len);
/*     */     }
/*     */     
/*     */     public int read() throws IOException
/*     */     {
/* 215 */       int n = read(this.oneByte, 0, 1);
/* 216 */       if (n == 1) {
/* 217 */         return this.oneByte[0] & 0xFF;
/*     */       }
/* 219 */       return n;
/*     */     }
/*     */     
/*     */     public long skip(long skip)
/*     */       throws IOException
/*     */     {
/* 225 */       long position = this.in.tell();
/* 226 */       long length = this.in.length();
/* 227 */       long remaining = length - position;
/* 228 */       if (remaining > skip) {
/* 229 */         this.in.seek(skip);
/* 230 */         return this.in.tell() - position;
/*     */       }
/* 232 */       this.in.seek(remaining);
/* 233 */       return this.in.tell() - position;
/*     */     }
/*     */     
/*     */     public void close()
/*     */       throws IOException
/*     */     {
/* 239 */       this.in.close();
/* 240 */       super.close();
/*     */     }
/*     */     
/*     */     public int available() throws IOException
/*     */     {
/* 245 */       long remaining = this.in.length() - this.in.tell();
/* 246 */       return remaining > 2147483647L ? Integer.MAX_VALUE : (int)remaining;
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\file\DataFileReader.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */