/*     */ package org.apache.avro.io;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import org.apache.avro.AvroTypeException;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.Schema.Field;
/*     */ import org.apache.avro.io.parsing.ResolvingGrammarGenerator;
/*     */ import org.apache.avro.io.parsing.SkipParser;
/*     */ import org.apache.avro.io.parsing.Symbol;
/*     */ import org.apache.avro.io.parsing.Symbol.Alternative;
/*     */ import org.apache.avro.io.parsing.Symbol.DefaultStartAction;
/*     */ import org.apache.avro.io.parsing.Symbol.EnumAdjustAction;
/*     */ import org.apache.avro.io.parsing.Symbol.ErrorAction;
/*     */ import org.apache.avro.io.parsing.Symbol.FieldOrderAction;
/*     */ import org.apache.avro.io.parsing.Symbol.ResolvingAction;
/*     */ import org.apache.avro.io.parsing.Symbol.SkipAction;
/*     */ import org.apache.avro.io.parsing.Symbol.UnionAdjustAction;
/*     */ import org.apache.avro.io.parsing.Symbol.WriterUnionAction;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class ResolvingDecoder
/*     */   extends ValidatingDecoder
/*     */ {
/*     */   private Decoder backup;
/*     */   
/*     */   ResolvingDecoder(Schema writer, Schema reader, Decoder in)
/*     */     throws IOException
/*     */   {
/*  46 */     this(resolve(writer, reader), in);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private ResolvingDecoder(Object resolver, Decoder in)
/*     */     throws IOException
/*     */   {
/*  59 */     super((Symbol)resolver, in);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static Object resolve(Schema writer, Schema reader)
/*     */     throws IOException
/*     */   {
/*  76 */     if (null == writer) {
/*  77 */       throw new NullPointerException("writer cannot be null!");
/*     */     }
/*  79 */     if (null == reader) {
/*  80 */       throw new NullPointerException("reader cannot be null!");
/*     */     }
/*  82 */     return new ResolvingGrammarGenerator().generate(writer, reader);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final Schema.Field[] readFieldOrder()
/*     */     throws IOException
/*     */   {
/* 127 */     return ((Symbol.FieldOrderAction)this.parser.advance(Symbol.FIELD_ACTION)).fields;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final void drain()
/*     */     throws IOException
/*     */   {
/* 147 */     this.parser.processImplicitActions();
/*     */   }
/*     */   
/*     */   public long readLong() throws IOException
/*     */   {
/* 152 */     Symbol actual = this.parser.advance(Symbol.LONG);
/* 153 */     if (actual == Symbol.INT)
/* 154 */       return this.in.readInt();
/* 155 */     if (actual == Symbol.DOUBLE) {
/* 156 */       return this.in.readDouble();
/*     */     }
/* 158 */     assert (actual == Symbol.LONG);
/* 159 */     return this.in.readLong();
/*     */   }
/*     */   
/*     */   public float readFloat()
/*     */     throws IOException
/*     */   {
/* 165 */     Symbol actual = this.parser.advance(Symbol.FLOAT);
/* 166 */     if (actual == Symbol.INT)
/* 167 */       return this.in.readInt();
/* 168 */     if (actual == Symbol.LONG) {
/* 169 */       return (float)this.in.readLong();
/*     */     }
/* 171 */     assert (actual == Symbol.FLOAT);
/* 172 */     return this.in.readFloat();
/*     */   }
/*     */   
/*     */   public double readDouble()
/*     */     throws IOException
/*     */   {
/* 178 */     Symbol actual = this.parser.advance(Symbol.DOUBLE);
/* 179 */     if (actual == Symbol.INT)
/* 180 */       return this.in.readInt();
/* 181 */     if (actual == Symbol.LONG)
/* 182 */       return this.in.readLong();
/* 183 */     if (actual == Symbol.FLOAT) {
/* 184 */       return this.in.readFloat();
/*     */     }
/* 186 */     assert (actual == Symbol.DOUBLE);
/* 187 */     return this.in.readDouble();
/*     */   }
/*     */   
/*     */   public int readEnum()
/*     */     throws IOException
/*     */   {
/* 193 */     this.parser.advance(Symbol.ENUM);
/* 194 */     Symbol.EnumAdjustAction top = (Symbol.EnumAdjustAction)this.parser.popSymbol();
/* 195 */     int n = this.in.readEnum();
/* 196 */     Object o = top.adjustments[n];
/* 197 */     if ((o instanceof Integer)) {
/* 198 */       return ((Integer)o).intValue();
/*     */     }
/* 200 */     throw new AvroTypeException((String)o);
/*     */   }
/*     */   
/*     */   public int readIndex()
/*     */     throws IOException
/*     */   {
/* 206 */     this.parser.advance(Symbol.UNION);
/* 207 */     Symbol.UnionAdjustAction top = (Symbol.UnionAdjustAction)this.parser.popSymbol();
/* 208 */     this.parser.pushSymbol(top.symToParse);
/* 209 */     return top.rindex;
/*     */   }
/*     */   
/*     */   public Symbol doAction(Symbol input, Symbol top) throws IOException
/*     */   {
/* 214 */     if ((top instanceof Symbol.FieldOrderAction))
/* 215 */       return input == Symbol.FIELD_ACTION ? top : null;
/* 216 */     if ((top instanceof Symbol.ResolvingAction)) {
/* 217 */       Symbol.ResolvingAction t = (Symbol.ResolvingAction)top;
/* 218 */       if (t.reader != input) {
/* 219 */         throw new AvroTypeException("Found " + t.reader + " while looking for " + input);
/*     */       }
/*     */       
/* 222 */       return t.writer;
/*     */     }
/* 224 */     if ((top instanceof Symbol.SkipAction)) {
/* 225 */       Symbol symToSkip = ((Symbol.SkipAction)top).symToSkip;
/* 226 */       this.parser.skipSymbol(symToSkip);
/* 227 */     } else if ((top instanceof Symbol.WriterUnionAction)) {
/* 228 */       Symbol.Alternative branches = (Symbol.Alternative)this.parser.popSymbol();
/* 229 */       this.parser.pushSymbol(branches.getSymbol(this.in.readIndex()));
/* 230 */     } else { if ((top instanceof Symbol.ErrorAction))
/* 231 */         throw new AvroTypeException(((Symbol.ErrorAction)top).msg);
/* 232 */       if ((top instanceof Symbol.DefaultStartAction)) {
/* 233 */         Symbol.DefaultStartAction dsa = (Symbol.DefaultStartAction)top;
/* 234 */         this.backup = this.in;
/* 235 */         this.in = DecoderFactory.get().binaryDecoder(dsa.contents, null);
/*     */       }
/* 237 */       else if (top == Symbol.DEFAULT_END_ACTION) {
/* 238 */         this.in = this.backup;
/*     */       } else {
/* 240 */         throw new AvroTypeException("Unknown action: " + top);
/*     */       } }
/* 242 */     return null;
/*     */   }
/*     */   
/*     */   public void skipAction() throws IOException
/*     */   {
/* 247 */     Symbol top = this.parser.popSymbol();
/* 248 */     if ((top instanceof Symbol.ResolvingAction)) {
/* 249 */       this.parser.pushSymbol(((Symbol.ResolvingAction)top).writer);
/* 250 */     } else if ((top instanceof Symbol.SkipAction)) {
/* 251 */       this.parser.pushSymbol(((Symbol.SkipAction)top).symToSkip);
/* 252 */     } else if ((top instanceof Symbol.WriterUnionAction)) {
/* 253 */       Symbol.Alternative branches = (Symbol.Alternative)this.parser.popSymbol();
/* 254 */       this.parser.pushSymbol(branches.getSymbol(this.in.readIndex()));
/* 255 */     } else { if ((top instanceof Symbol.ErrorAction))
/* 256 */         throw new AvroTypeException(((Symbol.ErrorAction)top).msg);
/* 257 */       if ((top instanceof Symbol.DefaultStartAction)) {
/* 258 */         Symbol.DefaultStartAction dsa = (Symbol.DefaultStartAction)top;
/* 259 */         this.backup = this.in;
/* 260 */         this.in = DecoderFactory.get().binaryDecoder(dsa.contents, null);
/*     */       }
/* 262 */       else if (top == Symbol.DEFAULT_END_ACTION) {
/* 263 */         this.in = this.backup;
/*     */       }
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\io\ResolvingDecoder.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */