/*     */ package org.codehaus.jackson.map.deser;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.util.HashMap;
/*     */ import java.util.concurrent.ConcurrentHashMap;
/*     */ import org.codehaus.jackson.JsonNode;
/*     */ import org.codehaus.jackson.JsonParser;
/*     */ import org.codehaus.jackson.JsonProcessingException;
/*     */ import org.codehaus.jackson.map.AbstractTypeResolver;
/*     */ import org.codehaus.jackson.map.AnnotationIntrospector;
/*     */ import org.codehaus.jackson.map.BeanProperty;
/*     */ import org.codehaus.jackson.map.ContextualDeserializer;
/*     */ import org.codehaus.jackson.map.ContextualKeyDeserializer;
/*     */ import org.codehaus.jackson.map.DeserializationConfig;
/*     */ import org.codehaus.jackson.map.DeserializationConfig.Feature;
/*     */ import org.codehaus.jackson.map.DeserializationContext;
/*     */ import org.codehaus.jackson.map.DeserializerFactory;
/*     */ import org.codehaus.jackson.map.DeserializerProvider;
/*     */ import org.codehaus.jackson.map.Deserializers;
/*     */ import org.codehaus.jackson.map.JsonDeserializer;
/*     */ import org.codehaus.jackson.map.JsonMappingException;
/*     */ import org.codehaus.jackson.map.KeyDeserializer;
/*     */ import org.codehaus.jackson.map.KeyDeserializers;
/*     */ import org.codehaus.jackson.map.ResolvableDeserializer;
/*     */ import org.codehaus.jackson.map.TypeDeserializer;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedClass;
/*     */ import org.codehaus.jackson.map.type.ArrayType;
/*     */ import org.codehaus.jackson.map.type.CollectionLikeType;
/*     */ import org.codehaus.jackson.map.type.CollectionType;
/*     */ import org.codehaus.jackson.map.type.MapLikeType;
/*     */ import org.codehaus.jackson.map.type.MapType;
/*     */ import org.codehaus.jackson.map.util.ClassUtil;
/*     */ import org.codehaus.jackson.type.JavaType;
/*     */ 
/*     */ public class StdDeserializerProvider extends DeserializerProvider
/*     */ {
/*  37 */   static final HashMap<JavaType, KeyDeserializer> _keyDeserializers = ;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  49 */   protected final ConcurrentHashMap<JavaType, JsonDeserializer<Object>> _cachedDeserializers = new ConcurrentHashMap(64, 0.75F, 2);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  57 */   protected final HashMap<JavaType, JsonDeserializer<Object>> _incompleteDeserializers = new HashMap(8);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected DeserializerFactory _factory;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public StdDeserializerProvider()
/*     */   {
/*  84 */     this(BeanDeserializerFactory.instance);
/*     */   }
/*     */   
/*  87 */   public StdDeserializerProvider(DeserializerFactory f) { this._factory = f; }
/*     */   
/*     */ 
/*     */   public DeserializerProvider withAdditionalDeserializers(Deserializers d)
/*     */   {
/*  92 */     this._factory = this._factory.withAdditionalDeserializers(d);
/*  93 */     return this;
/*     */   }
/*     */   
/*     */   public DeserializerProvider withAdditionalKeyDeserializers(KeyDeserializers d)
/*     */   {
/*  98 */     this._factory = this._factory.withAdditionalKeyDeserializers(d);
/*  99 */     return this;
/*     */   }
/*     */   
/*     */   public DeserializerProvider withDeserializerModifier(BeanDeserializerModifier modifier)
/*     */   {
/* 104 */     this._factory = this._factory.withDeserializerModifier(modifier);
/* 105 */     return this;
/*     */   }
/*     */   
/*     */   public DeserializerProvider withAbstractTypeResolver(AbstractTypeResolver resolver)
/*     */   {
/* 110 */     this._factory = this._factory.withAbstractTypeResolver(resolver);
/* 111 */     return this;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonDeserializer<Object> findValueDeserializer(DeserializationConfig config, JavaType propertyType, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 126 */     JsonDeserializer<Object> deser = _findCachedDeserializer(propertyType);
/* 127 */     if (deser != null)
/*     */     {
/* 129 */       if ((deser instanceof ContextualDeserializer)) {
/* 130 */         JsonDeserializer<?> d = ((ContextualDeserializer)deser).createContextual(config, property);
/* 131 */         deser = d;
/*     */       }
/* 133 */       return deser;
/*     */     }
/*     */     
/* 136 */     deser = _createAndCacheValueDeserializer(config, propertyType, property);
/* 137 */     if (deser == null)
/*     */     {
/*     */ 
/*     */ 
/*     */ 
/* 142 */       deser = _handleUnknownValueDeserializer(propertyType);
/*     */     }
/*     */     
/* 145 */     if ((deser instanceof ContextualDeserializer)) {
/* 146 */       JsonDeserializer<?> d = ((ContextualDeserializer)deser).createContextual(config, property);
/* 147 */       deser = d;
/*     */     }
/* 149 */     return deser;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public JsonDeserializer<Object> findTypedValueDeserializer(DeserializationConfig config, JavaType type, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 157 */     JsonDeserializer<Object> deser = findValueDeserializer(config, type, property);
/* 158 */     TypeDeserializer typeDeser = this._factory.findTypeDeserializer(config, type, property);
/* 159 */     if (typeDeser != null) {
/* 160 */       return new WrappedDeserializer(typeDeser, deser);
/*     */     }
/* 162 */     return deser;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public KeyDeserializer findKeyDeserializer(DeserializationConfig config, JavaType type, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 171 */     KeyDeserializer kd = this._factory.createKeyDeserializer(config, type, property);
/* 172 */     if (kd == null)
/*     */     {
/* 174 */       Class<?> raw = type.getRawClass();
/* 175 */       if ((raw == String.class) || (raw == Object.class)) {
/* 176 */         return null;
/*     */       }
/*     */       
/* 179 */       KeyDeserializer kdes = (KeyDeserializer)_keyDeserializers.get(type);
/* 180 */       if (kdes != null) {
/* 181 */         return kdes;
/*     */       }
/*     */       
/* 184 */       if (type.isEnumType()) {
/* 185 */         return StdKeyDeserializers.constructEnumKeyDeserializer(config, type);
/*     */       }
/*     */       
/* 188 */       kdes = StdKeyDeserializers.findStringBasedKeyDeserializer(config, type);
/* 189 */       if (kdes != null) {
/* 190 */         return kdes;
/*     */       }
/* 192 */       if (kd == null)
/*     */       {
/* 194 */         return _handleUnknownKeyDeserializer(type);
/*     */       }
/*     */     }
/*     */     
/* 198 */     if ((kd instanceof ContextualKeyDeserializer)) {
/* 199 */       kd = ((ContextualKeyDeserializer)kd).createContextual(config, property);
/*     */     }
/* 201 */     return kd;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public boolean hasValueDeserializerFor(DeserializationConfig config, JavaType type)
/*     */   {
/* 214 */     JsonDeserializer<Object> deser = _findCachedDeserializer(type);
/* 215 */     if (deser == null) {
/*     */       try {
/* 217 */         deser = _createAndCacheValueDeserializer(config, type, null);
/*     */       } catch (Exception e) {
/* 219 */         return false;
/*     */       }
/*     */     }
/* 222 */     return deser != null;
/*     */   }
/*     */   
/*     */   public int cachedDeserializersCount()
/*     */   {
/* 227 */     return this._cachedDeserializers.size();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void flushCachedDeserializers()
/*     */   {
/* 241 */     this._cachedDeserializers.clear();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonDeserializer<Object> _findCachedDeserializer(JavaType type)
/*     */   {
/* 252 */     return (JsonDeserializer)this._cachedDeserializers.get(type);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonDeserializer<Object> _createAndCacheValueDeserializer(DeserializationConfig config, JavaType type, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 271 */     synchronized (this._incompleteDeserializers)
/*     */     {
/* 273 */       JsonDeserializer<Object> deser = _findCachedDeserializer(type);
/* 274 */       if (deser != null) {
/* 275 */         return deser;
/*     */       }
/* 277 */       int count = this._incompleteDeserializers.size();
/*     */       
/* 279 */       if (count > 0) {
/* 280 */         deser = (JsonDeserializer)this._incompleteDeserializers.get(type);
/* 281 */         if (deser != null) {
/* 282 */           return deser;
/*     */         }
/*     */       }
/*     */       try
/*     */       {
/* 287 */         JsonDeserializer localJsonDeserializer = _createAndCache2(config, type, property);
/*     */         
/*     */ 
/* 290 */         if ((count == 0) && (this._incompleteDeserializers.size() > 0))
/* 291 */           this._incompleteDeserializers.clear(); return localJsonDeserializer;
/*     */       }
/*     */       finally
/*     */       {
/* 290 */         if ((count == 0) && (this._incompleteDeserializers.size() > 0)) {
/* 291 */           this._incompleteDeserializers.clear();
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   protected JsonDeserializer<Object> _createAndCache2(DeserializationConfig config, JavaType type, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/*     */     JsonDeserializer<Object> deser;
/*     */     
/*     */ 
/*     */     try
/*     */     {
/* 307 */       deser = _createDeserializer(config, type, property);
/*     */ 
/*     */     }
/*     */     catch (IllegalArgumentException iae)
/*     */     {
/* 312 */       throw new JsonMappingException(iae.getMessage(), null, iae);
/*     */     }
/* 314 */     if (deser == null) {
/* 315 */       return null;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 321 */     boolean isResolvable = deser instanceof ResolvableDeserializer;
/* 322 */     boolean addToCache = deser.getClass() == BeanDeserializer.class;
/* 323 */     if (!addToCache)
/*     */     {
/* 325 */       if (config.isEnabled(DeserializationConfig.Feature.USE_ANNOTATIONS)) {
/* 326 */         AnnotationIntrospector aintr = config.getAnnotationIntrospector();
/*     */         
/* 328 */         AnnotatedClass ac = AnnotatedClass.construct(deser.getClass(), aintr, null);
/* 329 */         Boolean cacheAnn = aintr.findCachability(ac);
/* 330 */         if (cacheAnn != null) {
/* 331 */           addToCache = cacheAnn.booleanValue();
/*     */         }
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 347 */     if (isResolvable) {
/* 348 */       this._incompleteDeserializers.put(type, deser);
/* 349 */       _resolveDeserializer(config, (ResolvableDeserializer)deser);
/* 350 */       this._incompleteDeserializers.remove(type);
/*     */     }
/* 352 */     if (addToCache) {
/* 353 */       this._cachedDeserializers.put(type, deser);
/*     */     }
/* 355 */     return deser;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonDeserializer<Object> _createDeserializer(DeserializationConfig config, JavaType type, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 366 */     if (type.isEnumType()) {
/* 367 */       return this._factory.createEnumDeserializer(config, this, type, property);
/*     */     }
/* 369 */     if (type.isContainerType()) {
/* 370 */       if (type.isArrayType()) {
/* 371 */         return this._factory.createArrayDeserializer(config, this, (ArrayType)type, property);
/*     */       }
/*     */       
/* 374 */       if (type.isMapLikeType()) {
/* 375 */         MapLikeType mlt = (MapLikeType)type;
/* 376 */         if (mlt.isTrueMapType()) {
/* 377 */           return this._factory.createMapDeserializer(config, this, (MapType)mlt, property);
/*     */         }
/*     */         
/* 380 */         return this._factory.createMapLikeDeserializer(config, this, mlt, property);
/*     */       }
/*     */       
/* 383 */       if (type.isCollectionLikeType()) {
/* 384 */         CollectionLikeType clt = (CollectionLikeType)type;
/* 385 */         if (clt.isTrueCollectionType()) {
/* 386 */           return this._factory.createCollectionDeserializer(config, this, (CollectionType)clt, property);
/*     */         }
/*     */         
/* 389 */         return this._factory.createCollectionLikeDeserializer(config, this, clt, property);
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 395 */     if (JsonNode.class.isAssignableFrom(type.getRawClass())) {
/* 396 */       return this._factory.createTreeDeserializer(config, this, type, property);
/*     */     }
/* 398 */     return this._factory.createBeanDeserializer(config, this, type, property);
/*     */   }
/*     */   
/*     */   protected void _resolveDeserializer(DeserializationConfig config, ResolvableDeserializer ser)
/*     */     throws JsonMappingException
/*     */   {
/* 404 */     ser.resolve(config, this);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonDeserializer<Object> _handleUnknownValueDeserializer(JavaType type)
/*     */     throws JsonMappingException
/*     */   {
/* 419 */     Class<?> rawClass = type.getRawClass();
/* 420 */     if (!ClassUtil.isConcrete(rawClass)) {
/* 421 */       throw new JsonMappingException("Can not find a Value deserializer for abstract type " + type);
/*     */     }
/* 423 */     throw new JsonMappingException("Can not find a Value deserializer for type " + type);
/*     */   }
/*     */   
/*     */   protected KeyDeserializer _handleUnknownKeyDeserializer(JavaType type)
/*     */     throws JsonMappingException
/*     */   {
/* 429 */     throw new JsonMappingException("Can not find a (Map) Key deserializer for type " + type);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected static final class WrappedDeserializer
/*     */     extends JsonDeserializer<Object>
/*     */   {
/*     */     final TypeDeserializer _typeDeserializer;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     final JsonDeserializer<Object> _deserializer;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public WrappedDeserializer(TypeDeserializer typeDeser, JsonDeserializer<Object> deser)
/*     */     {
/* 452 */       this._typeDeserializer = typeDeser;
/* 453 */       this._deserializer = deser;
/*     */     }
/*     */     
/*     */ 
/*     */     public Object deserialize(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 460 */       return this._deserializer.deserializeWithType(jp, ctxt, this._typeDeserializer);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     public Object deserializeWithType(JsonParser jp, DeserializationContext ctxt, TypeDeserializer typeDeserializer)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 469 */       throw new IllegalStateException("Type-wrapped deserializer's deserializeWithType should never get called");
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\deser\StdDeserializerProvider.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */