/*     */ package org.codehaus.janino;
/*     */ 
/*     */ import java.util.HashMap;
/*     */ import java.util.Iterator;
/*     */ import java.util.Map;
/*     */ import java.util.Map.Entry;
/*     */ import java.util.Set;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public final class Descriptor
/*     */ {
/*     */   public static final String VOID_ = "V";
/*     */   public static final String BYTE_ = "B";
/*     */   public static final String CHAR_ = "C";
/*     */   public static final String DOUBLE_ = "D";
/*     */   public static final String FLOAT_ = "F";
/*     */   public static final String INT_ = "I";
/*     */   public static final String LONG_ = "J";
/*     */   public static final String SHORT_ = "S";
/*     */   public static final String BOOLEAN_ = "Z";
/*     */   public static final String OBJECT = "Ljava/lang/Object;";
/*     */   public static final String STRING = "Ljava/lang/String;";
/*     */   public static final String STRING_BUFFER = "Ljava/lang/StringBuffer;";
/*     */   public static final String STRING_BUILDER = "Ljava/lang/StringBuilder;";
/*     */   public static final String CLASS = "Ljava/lang/Class;";
/*     */   public static final String THROWABLE = "Ljava/lang/Throwable;";
/*     */   public static final String RUNTIME_EXCEPTION = "Ljava/lang/RuntimeException;";
/*     */   public static final String ERROR = "Ljava/lang/Error;";
/*     */   public static final String CLONEABLE = "Ljava/lang/Cloneable;";
/*     */   public static final String SERIALIZABLE = "Ljava/io/Serializable;";
/*     */   public static final String BOOLEAN = "Ljava/lang/Boolean;";
/*     */   public static final String BYTE = "Ljava/lang/Byte;";
/*     */   public static final String CHARACTER = "Ljava/lang/Character;";
/*     */   public static final String SHORT = "Ljava/lang/Short;";
/*     */   public static final String INTEGER = "Ljava/lang/Integer;";
/*     */   public static final String LONG = "Ljava/lang/Long;";
/*     */   public static final String FLOAT = "Ljava/lang/Float;";
/*     */   public static final String DOUBLE = "Ljava/lang/Double;";
/*     */   
/*     */   public static boolean isReference(String d)
/*     */   {
/*  48 */     return d.length() > 1;
/*     */   }
/*     */   
/*  51 */   public static boolean isClassOrInterfaceReference(String d) { return d.charAt(0) == 'L'; }
/*     */   
/*     */ 
/*  54 */   public static boolean isArrayReference(String d) { return d.charAt(0) == '['; }
/*     */   
/*     */   public static String getComponentDescriptor(String d) {
/*  57 */     if (d.charAt(0) != '[') {
/*  58 */       throw new JaninoRuntimeException("Cannot determine component descriptor from non-array descriptor \"" + d + "\"");
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*  64 */     return d.substring(1);
/*     */   }
/*     */   
/*  67 */   public static short size(String d) { if (d.equals("V")) return 0;
/*  68 */     if (hasSize1(d)) return 1;
/*  69 */     if (hasSize2(d)) return 2;
/*  70 */     throw new JaninoRuntimeException("No size defined for type \"" + toString(d) + "\"");
/*     */   }
/*     */   
/*  73 */   public static boolean hasSize1(String d) { if (d.length() == 1) return "BCFISZ".indexOf(d) != -1;
/*  74 */     return isReference(d);
/*     */   }
/*     */   
/*  77 */   public static boolean hasSize2(String d) { return (d.equals("J")) || (d.equals("D")); }
/*     */   
/*     */ 
/*     */   public static String toString(String d)
/*     */   {
/*  82 */     int idx = 0;
/*  83 */     StringBuffer sb = new StringBuffer();
/*  84 */     if (d.charAt(0) == '(') {
/*  85 */       idx++;
/*  86 */       sb.append("(");
/*  87 */       while ((idx < d.length()) && (d.charAt(idx) != ')')) {
/*  88 */         if (idx != 1) sb.append(", ");
/*  89 */         idx = toString(d, idx, sb);
/*     */       }
/*  91 */       if (idx >= d.length()) throw new JaninoRuntimeException("Invalid descriptor \"" + d + "\"");
/*  92 */       sb.append(") => ");
/*  93 */       idx++;
/*     */     }
/*  95 */     toString(d, idx, sb);
/*  96 */     return sb.toString();
/*     */   }
/*     */   
/*  99 */   private static int toString(String d, int idx, StringBuffer sb) { int dimensions = 0;
/* 100 */     while ((idx < d.length()) && (d.charAt(idx) == '[')) {
/* 101 */       dimensions++;
/* 102 */       idx++;
/*     */     }
/* 104 */     if (idx >= d.length()) throw new JaninoRuntimeException("Invalid descriptor \"" + d + "\"");
/* 105 */     switch (d.charAt(idx))
/*     */     {
/*     */     case 'L': 
/* 108 */       int idx2 = d.indexOf(';', idx);
/* 109 */       if (idx2 == -1) throw new JaninoRuntimeException("Invalid descriptor \"" + d + "\"");
/* 110 */       sb.append(d.substring(idx + 1, idx2).replace('/', '.'));
/* 111 */       idx = idx2;
/*     */       
/* 113 */       break;
/* 114 */     case 'V':  sb.append("void"); break;
/* 115 */     case 'B':  sb.append("byte"); break;
/* 116 */     case 'C':  sb.append("char"); break;
/* 117 */     case 'D':  sb.append("double"); break;
/* 118 */     case 'F':  sb.append("float"); break;
/* 119 */     case 'I':  sb.append("int"); break;
/* 120 */     case 'J':  sb.append("long"); break;
/* 121 */     case 'S':  sb.append("short"); break;
/* 122 */     case 'Z':  sb.append("boolean"); break;
/*     */     case 'E': case 'G': case 'H': case 'K': case 'M': case 'N': case 'O': case 'P': case 'Q': case 'R': case 'T': case 'U': case 'W': case 'X': case 'Y': default: 
/* 124 */       throw new JaninoRuntimeException("Invalid descriptor \"" + d + "\"");
/*     */     }
/* 126 */     for (; dimensions > 0; dimensions--) sb.append("[]");
/* 127 */     return idx + 1;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public static String fromClassName(String className)
/*     */   {
/* 135 */     String res = (String)classNameToDescriptor.get(className);
/* 136 */     if (res != null) return res;
/* 137 */     if (className.startsWith("[")) return className.replace('.', '/');
/* 138 */     return 'L' + className.replace('.', '/') + ';';
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static String fromInternalForm(String internalForm)
/*     */   {
/* 147 */     if (internalForm.charAt(0) == '[') return internalForm;
/* 148 */     return 'L' + internalForm + ';';
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public static String toClassName(String d)
/*     */   {
/* 156 */     String res = (String)descriptorToClassName.get(d);
/* 157 */     if (res != null) { return res;
/*     */     }
/* 159 */     char firstChar = d.charAt(0);
/* 160 */     if ((firstChar == 'L') && (d.endsWith(";")))
/*     */     {
/* 162 */       return d.substring(1, d.length() - 1).replace('/', '.');
/*     */     }
/* 164 */     if (firstChar == '[')
/*     */     {
/* 166 */       return d.replace('/', '.');
/*     */     }
/* 168 */     throw new JaninoRuntimeException("(Invalid field descriptor \"" + d + "\")");
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public static String toInternalForm(String d)
/*     */   {
/* 175 */     if (d.charAt(0) != 'L') {
/* 176 */       throw new JaninoRuntimeException("Attempt to convert non-class descriptor \"" + d + "\" into internal form");
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 182 */     return d.substring(1, d.length() - 1);
/*     */   }
/*     */   
/*     */   public static boolean isPrimitive(String d) {
/* 186 */     return (d.length() == 1) && ("VBCDFIJSZ".indexOf(d.charAt(0)) != -1);
/*     */   }
/*     */   
/*     */   public static boolean isPrimitiveNumeric(String d) {
/* 190 */     return (d.length() == 1) && ("BDFIJSC".indexOf(d.charAt(0)) != -1);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static String getPackageName(String d)
/*     */   {
/* 199 */     if (d.charAt(0) != 'L') {
/* 200 */       throw new JaninoRuntimeException("Attempt to get package name of non-class descriptor \"" + d + "\"");
/*     */     }
/* 202 */     int idx = d.lastIndexOf('/');
/* 203 */     return idx == -1 ? null : d.substring(1, idx).replace('/', '.');
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public static boolean areInSamePackage(String d1, String d2)
/*     */   {
/* 210 */     String packageName1 = getPackageName(d1);
/* 211 */     String packageName2 = getPackageName(d2);
/* 212 */     return packageName1 == null ? false : packageName2 == null ? true : packageName1.equals(packageName2);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 243 */   private static final Map classNameToDescriptor = new HashMap();
/* 244 */   private static final Map descriptorToClassName = new HashMap();
/*     */   
/* 246 */   static { descriptorToClassName.put("V", "void");
/* 247 */     descriptorToClassName.put("B", "byte");
/* 248 */     descriptorToClassName.put("C", "char");
/* 249 */     descriptorToClassName.put("D", "double");
/* 250 */     descriptorToClassName.put("F", "float");
/* 251 */     descriptorToClassName.put("I", "int");
/* 252 */     descriptorToClassName.put("J", "long");
/* 253 */     descriptorToClassName.put("S", "short");
/* 254 */     descriptorToClassName.put("Z", "boolean");
/* 255 */     descriptorToClassName.put("Ljava/lang/Object;", "java.lang.Object");
/* 256 */     descriptorToClassName.put("Ljava/lang/String;", "java.lang.String");
/* 257 */     descriptorToClassName.put("Ljava/lang/StringBuffer;", "java.lang.StringBuffer");
/* 258 */     descriptorToClassName.put("Ljava/lang/StringBuilder;", "java.lang.StringBuilder");
/* 259 */     descriptorToClassName.put("Ljava/lang/Class;", "java.lang.Class");
/* 260 */     descriptorToClassName.put("Ljava/lang/Throwable;", "java.lang.Throwable");
/* 261 */     descriptorToClassName.put("Ljava/lang/RuntimeException;", "java.lang.RuntimeException");
/* 262 */     descriptorToClassName.put("Ljava/lang/Error;", "java.lang.Error");
/* 263 */     descriptorToClassName.put("Ljava/lang/Cloneable;", "java.lang.Cloneable");
/* 264 */     descriptorToClassName.put("Ljava/io/Serializable;", "java.io.Serializable");
/* 265 */     descriptorToClassName.put("Ljava/lang/Boolean;", "java.lang.Boolean");
/* 266 */     descriptorToClassName.put("Ljava/lang/Byte;", "java.lang.Byte");
/* 267 */     descriptorToClassName.put("Ljava/lang/Character;", "java.lang.Character");
/* 268 */     descriptorToClassName.put("Ljava/lang/Short;", "java.lang.Short");
/* 269 */     descriptorToClassName.put("Ljava/lang/Integer;", "java.lang.Integer");
/* 270 */     descriptorToClassName.put("Ljava/lang/Long;", "java.lang.Long");
/* 271 */     descriptorToClassName.put("Ljava/lang/Float;", "java.lang.Float");
/* 272 */     descriptorToClassName.put("Ljava/lang/Double;", "java.lang.Double");
/*     */     
/* 274 */     for (Iterator it = descriptorToClassName.entrySet().iterator(); it.hasNext();) {
/* 275 */       Map.Entry e = (Map.Entry)it.next();
/* 276 */       classNameToDescriptor.put(e.getValue(), e.getKey());
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\janino\Descriptor.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       0.7.1
 */