/*     */ package org.codehaus.janino.tools;
/*     */ 
/*     */ import java.io.BufferedReader;
/*     */ import java.io.FileReader;
/*     */ import java.io.IOException;
/*     */ import java.io.PrintStream;
/*     */ import java.text.FieldPosition;
/*     */ import java.text.MessageFormat;
/*     */ import java.util.ArrayList;
/*     */ import java.util.Arrays;
/*     */ import java.util.Comparator;
/*     */ import java.util.HashMap;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.StringTokenizer;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public final class HprofScrubber
/*     */ {
/*     */   private static class Site
/*     */   {
/*     */     public final int allocatedBytes;
/*     */     public final int allocatedObjects;
/*     */     public final int traceNumber;
/*     */     public final String className;
/*     */     
/*     */     public Site(int allocatedBytes, int allocatedObjects, int traceNumber, String className)
/*     */     {
/*  47 */       this.allocatedBytes = allocatedBytes;
/*  48 */       this.allocatedObjects = allocatedObjects;
/*  49 */       this.traceNumber = traceNumber;
/*  50 */       this.className = className;
/*     */     }
/*     */   }
/*     */   
/*     */   private static class Sample {
/*     */     public final int count;
/*     */     public final int traceNumber;
/*     */     
/*  58 */     public Sample(int count, int traceNumber) { this.count = count;
/*  59 */       this.traceNumber = traceNumber;
/*     */     }
/*     */   }
/*     */   
/*     */   public static void main(String[] args) throws Exception {
/*  64 */     String fileName = args.length == 0 ? "java.hprof.txt" : args[0];
/*     */     
/*  66 */     BufferedReader br = new BufferedReader(new FileReader(fileName));
/*     */     try {
/*  68 */       Map traces = new HashMap();
/*  69 */       List sites = new ArrayList();
/*  70 */       List samples = new ArrayList();
/*     */       
/*  72 */       String s = br.readLine();
/*  73 */       while (s != null) {
/*  74 */         if (s.startsWith("SITES BEGIN")) {
/*  75 */           br.readLine();
/*  76 */           br.readLine();
/*     */           for (;;) {
/*  78 */             s = br.readLine();
/*  79 */             if (s.startsWith("SITES END")) break;
/*  80 */             StringTokenizer st = new StringTokenizer(s);
/*  81 */             st.nextToken();
/*  82 */             st.nextToken();
/*  83 */             st.nextToken();
/*  84 */             st.nextToken();
/*  85 */             st.nextToken();
/*  86 */             sites.add(new Site(Integer.parseInt(st.nextToken()), Integer.parseInt(st.nextToken()), Integer.parseInt(st.nextToken()), st.nextToken()));
/*     */           }
/*     */         }
/*     */         
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  94 */         if ((s.startsWith("TRACE ")) && (s.endsWith(":"))) {
/*  95 */           int traceNumber = Integer.parseInt(s.substring(6, s.length() - 1));
/*  96 */           List l = new ArrayList();
/*     */           for (;;) {
/*  98 */             s = br.readLine();
/*  99 */             if (!s.startsWith("\t")) break;
/* 100 */             l.add(s.substring(1));
/*     */           }
/* 102 */           traces.put(new Integer(traceNumber), l.toArray(new String[l.size()]));
/*     */ 
/*     */         }
/*     */         else
/*     */         {
/* 107 */           if (s.startsWith("CPU SAMPLES BEGIN")) {
/* 108 */             br.readLine();
/*     */             for (;;) {
/* 110 */               s = br.readLine();
/* 111 */               if (s.startsWith("CPU SAMPLES END")) break;
/* 112 */               StringTokenizer st = new StringTokenizer(s);
/* 113 */               st.nextToken();
/* 114 */               st.nextToken();
/* 115 */               st.nextToken();
/* 116 */               int count = Integer.parseInt(st.nextToken());
/* 117 */               if (count != 0) {
/* 118 */                 int trace = Integer.parseInt(st.nextToken());
/* 119 */                 samples.add(new Sample(count, trace));
/*     */               }
/*     */             } }
/* 122 */           s = br.readLine();
/*     */         }
/*     */       }
/*     */       
/* 126 */       dumpSites((Site[])sites.toArray(new Site[sites.size()]), traces);
/*     */       
/* 128 */       dumpSamples((Sample[])samples.toArray(new Sample[samples.size()]), traces); return;
/*     */     } finally {
/*     */       try {
/* 131 */         br.close();
/*     */       } catch (IOException e) {}
/*     */     }
/*     */   }
/*     */   
/* 136 */   private static void dumpSites(Site[] ss, Map traces) { Arrays.sort(ss, new Comparator() {
/*     */       public int compare(Object o1, Object o2) {
/* 138 */         return ((HprofScrubber.Site)o2).allocatedBytes - ((HprofScrubber.Site)o1).allocatedBytes;
/*     */       }
/*     */       
/* 141 */     });
/* 142 */     int totalAllocatedBytes = 0;int totalAllocatedObjects = 0;
/* 143 */     for (int i = 0; i < ss.length; i++) {
/* 144 */       Site site = ss[i];
/* 145 */       totalAllocatedBytes += site.allocatedBytes;
/* 146 */       totalAllocatedObjects += site.allocatedObjects;
/*     */     }
/*     */     
/* 149 */     System.out.println("          percent          alloc'ed");
/* 150 */     System.out.println("rank   self  accum      bytes  objects  class name");
/* 151 */     System.out.println("Total:              " + totalAllocatedBytes + "  " + totalAllocatedObjects);
/*     */     
/* 153 */     double accumulatedPercentage = 0.0D;
/* 154 */     MessageFormat mf = new MessageFormat("{0,number,00000} {1,number,00.00}% {2,number,00.00}% {3,number,000000000} {4,number,000000000} {5}");
/*     */     
/*     */ 
/* 157 */     for (int i = 0; i < ss.length; i++) {
/* 158 */       Site site = ss[i];
/* 159 */       double selfPercentage = 100.0D * (site.allocatedBytes / totalAllocatedBytes);
/* 160 */       accumulatedPercentage += selfPercentage;
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 174 */       System.out.println(mf.format(new Object[] { new Integer(i + 1), new Double(selfPercentage), new Double(accumulatedPercentage), new Integer(site.allocatedBytes), new Integer(site.allocatedObjects), site.className }, new StringBuffer(), new FieldPosition(0)));
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 186 */       String[] stackFrames = (String[])traces.get(new Integer(site.traceNumber));
/* 187 */       if (stackFrames != null) {
/* 188 */         for (int j = 0; j < stackFrames.length; j++) {
/* 189 */           System.out.println("                           " + stackFrames[j]);
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   private static void dumpSamples(Sample[] ss, Map traces) {
/* 196 */     int totalCount = 0;
/* 197 */     for (int i = 0; i < ss.length; i++) { totalCount += ss[i].count;
/*     */     }
/* 199 */     System.out.println("          percent");
/* 200 */     System.out.println("rank   self  accum      count");
/* 201 */     System.out.println("Total:              " + totalCount);
/*     */     
/* 203 */     double accumulatedPercentage = 0.0D;
/* 204 */     MessageFormat mf = new MessageFormat("{0,number,00000} {1,number,00.00}% {2,number,00.00}% {3,number,000000000}");
/*     */     
/*     */ 
/* 207 */     for (int i = 0; i < ss.length; i++) {
/* 208 */       Sample sample = ss[i];
/* 209 */       double selfPercentage = 100.0D * (sample.count / totalCount);
/* 210 */       accumulatedPercentage += selfPercentage;
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 222 */       System.out.println(mf.format(new Object[] { new Integer(i + 1), new Double(selfPercentage), new Double(accumulatedPercentage), new Integer(sample.count) }, new StringBuffer(), new FieldPosition(0)));
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 232 */       String[] stackFrames = (String[])traces.get(new Integer(sample.traceNumber));
/* 233 */       if (stackFrames != null) {
/* 234 */         for (int j = 0; j < stackFrames.length; j++) {
/* 235 */           System.out.println("                           " + stackFrames[j]);
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\janino\tools\HprofScrubber.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       0.7.1
 */