package prefuse.action.layout.graph;

import java.awt.geom.Point2D;
import java.util.Iterator;
import prefuse.Visualization;
import prefuse.data.Graph;
import prefuse.data.Schema;
import prefuse.data.tuple.TupleSet;
import prefuse.visual.NodeItem;

public class BalloonTreeLayout
  extends TreeLayout
{
  private int m_minRadius = 2;
  public static final String PARAMS = "_balloonTreeLayoutParams";
  public static final Schema PARAMS_SCHEMA = new Schema();
  
  public BalloonTreeLayout(String paramString)
  {
    this(paramString, 2);
  }
  
  public BalloonTreeLayout(String paramString, int paramInt)
  {
    super(paramString);
  }
  
  public int getMinRadius()
  {
    return this.m_minRadius;
  }
  
  public void setMinRadius(int paramInt)
  {
    this.m_minRadius = paramInt;
  }
  
  public void run(double paramDouble)
  {
    Graph localGraph = (Graph)this.m_vis.getGroup(this.m_group);
    initSchema(localGraph.getNodes());
    Point2D localPoint2D = getLayoutAnchor();
    NodeItem localNodeItem = getLayoutRoot();
    layout(localNodeItem, localPoint2D.getX(), localPoint2D.getY());
  }
  
  private void layout(NodeItem paramNodeItem, double paramDouble1, double paramDouble2)
  {
    firstWalk(paramNodeItem);
    secondWalk(paramNodeItem, null, paramDouble1, paramDouble2, 1.0D, 0.0D);
  }
  
  private void firstWalk(NodeItem paramNodeItem)
  {
    Params localParams1 = getParams(paramNodeItem);
    localParams1.d = 0;
    double d = 0.0D;
    Iterator localIterator = paramNodeItem.children();
    while (localIterator.hasNext())
    {
      NodeItem localNodeItem = (NodeItem)localIterator.next();
      if (localNodeItem.isVisible())
      {
        firstWalk(localNodeItem);
        Params localParams2 = getParams(localNodeItem);
        localParams1.d = Math.max(localParams1.d, localParams2.r);
        localParams2.a = Math.atan(localParams2.r / (localParams1.d + localParams2.r));
        d += localParams2.a;
      }
    }
    adjustChildren(localParams1, d);
    setRadius(localParams1);
  }
  
  private void adjustChildren(Params paramParams, double paramDouble)
  {
    if (paramDouble > 3.141592653589793D)
    {
      paramParams.c = (3.141592653589793D / paramDouble);
      paramParams.f = 0.0D;
    }
    else
    {
      paramParams.c = 1.0D;
      paramParams.f = (3.141592653589793D - paramDouble);
    }
  }
  
  private void setRadius(Params paramParams)
  {
    paramParams.r = (Math.max(paramParams.d, this.m_minRadius) + 2 * paramParams.d);
  }
  
  private void secondWalk(NodeItem paramNodeItem1, NodeItem paramNodeItem2, double paramDouble1, double paramDouble2, double paramDouble3, double paramDouble4)
  {
    setX(paramNodeItem1, paramNodeItem2, paramDouble1);
    setY(paramNodeItem1, paramNodeItem2, paramDouble2);
    Params localParams1 = getParams(paramNodeItem1);
    int i = 0;
    Iterator localIterator = paramNodeItem1.children();
    while (localIterator.hasNext())
    {
      NodeItem localNodeItem1 = (NodeItem)localIterator.next();
      if (localNodeItem1.isVisible()) {
        i++;
      }
    }
    double d1 = paramDouble3 * localParams1.d;
    double d2 = paramDouble4 + 3.141592653589793D;
    double d3 = i == 0 ? 0.0D : localParams1.f / i;
    double d4 = 0.0D;
    localIterator = paramNodeItem1.children();
    while (localIterator.hasNext())
    {
      NodeItem localNodeItem2 = (NodeItem)localIterator.next();
      if (localNodeItem2.isVisible())
      {
        Params localParams2 = getParams(localNodeItem2);
        double d5 = localParams1.c * localParams2.a;
        double d6 = localParams1.d * Math.tan(d5) / (1.0D - Math.tan(d5));
        d2 += d4 + d5 + d3;
        double d7 = (paramDouble3 * d6 + d1) * Math.cos(d2);
        double d8 = (paramDouble3 * d6 + d1) * Math.sin(d2);
        d4 = d5;
        secondWalk(localNodeItem2, paramNodeItem1, paramDouble1 + d7, paramDouble2 + d8, paramDouble3 * localParams1.c, d2);
      }
    }
  }
  
  private void initSchema(TupleSet paramTupleSet)
  {
    try
    {
      paramTupleSet.addColumns(PARAMS_SCHEMA);
    }
    catch (IllegalArgumentException localIllegalArgumentException) {}
  }
  
  private Params getParams(NodeItem paramNodeItem)
  {
    Params localParams = (Params)paramNodeItem.get("_balloonTreeLayoutParams");
    if (localParams == null)
    {
      localParams = new Params();
      paramNodeItem.set("_balloonTreeLayoutParams", localParams);
    }
    return localParams;
  }
  
  static
  {
    PARAMS_SCHEMA.addColumn("_balloonTreeLayoutParams", Params.class);
  }
  
  public static class Params
  {
    public int d;
    public int r;
    public double rx;
    public double ry;
    public double a;
    public double c;
    public double f;
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\action\layout\graph\BalloonTreeLayout.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */