package prefuse.data.column;

import java.util.BitSet;
import prefuse.data.DataReadOnlyException;
import prefuse.data.DataTypeException;

public class BooleanColumn
  extends AbstractColumn
{
  private BitSet m_bits;
  private int m_size;
  
  public BooleanColumn()
  {
    this(0, 10, false);
  }
  
  public BooleanColumn(int paramInt)
  {
    this(paramInt, paramInt, false);
  }
  
  public BooleanColumn(int paramInt1, int paramInt2, boolean paramBoolean)
  {
    super(Boolean.TYPE, new Boolean(paramBoolean));
    if (paramInt2 < paramInt1) {
      throw new IllegalArgumentException("Capacity value can not be less than the row count.");
    }
    this.m_bits = new BitSet(paramInt2);
    this.m_bits.set(0, paramInt2, paramBoolean);
    this.m_size = paramInt1;
  }
  
  public int getRowCount()
  {
    return this.m_size;
  }
  
  public void setMaximumRow(int paramInt)
  {
    if (paramInt > this.m_size) {
      this.m_bits.set(this.m_size, paramInt, ((Boolean)this.m_defaultValue).booleanValue());
    }
    this.m_size = paramInt;
  }
  
  public Object get(int paramInt)
  {
    return new Boolean(getBoolean(paramInt));
  }
  
  public void set(Object paramObject, int paramInt)
    throws DataTypeException
  {
    if (this.m_readOnly) {
      throw new DataReadOnlyException();
    }
    if (paramObject != null)
    {
      if ((paramObject instanceof Boolean)) {
        setBoolean(((Boolean)paramObject).booleanValue(), paramInt);
      } else if ((paramObject instanceof String)) {
        setString((String)paramObject, paramInt);
      } else {
        throw new DataTypeException(paramObject.getClass());
      }
    }
    else {
      throw new DataTypeException("Column does not accept null values");
    }
  }
  
  public boolean getBoolean(int paramInt)
    throws DataTypeException
  {
    if ((paramInt < 0) || (paramInt > this.m_size)) {
      throw new IllegalArgumentException("Row index out of bounds: " + paramInt);
    }
    return this.m_bits.get(paramInt);
  }
  
  public void setBoolean(boolean paramBoolean, int paramInt)
    throws DataTypeException
  {
    if (this.m_readOnly) {
      throw new DataReadOnlyException();
    }
    if ((paramInt < 0) || (paramInt >= this.m_size)) {
      throw new IllegalArgumentException("Row index out of bounds: " + paramInt);
    }
    boolean bool = this.m_bits.get(paramInt);
    if (bool == paramBoolean) {
      return;
    }
    this.m_bits.set(paramInt, paramBoolean);
    fireColumnEvent(paramInt, bool);
  }
  
  public String getString(int paramInt)
    throws DataTypeException
  {
    return String.valueOf(getBoolean(paramInt));
  }
  
  public void setString(String paramString, int paramInt)
    throws DataTypeException
  {
    boolean bool;
    if ("true".equalsIgnoreCase(paramString)) {
      bool = true;
    } else if ("false".equalsIgnoreCase(paramString)) {
      bool = false;
    } else {
      throw new IllegalArgumentException("Input string does not represent a boolean value.");
    }
    setBoolean(bool, paramInt);
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\column\BooleanColumn.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */