package prefuse.data.expression;

import prefuse.data.Schema;
import prefuse.data.Tuple;
import prefuse.util.TypeLib;

public class IfExpression
  extends AbstractExpression
{
  private Predicate m_test;
  private Expression m_then;
  private Expression m_else;
  
  public IfExpression(Predicate paramPredicate, Expression paramExpression1, Expression paramExpression2)
  {
    this.m_test = paramPredicate;
    this.m_then = paramExpression1;
    this.m_else = paramExpression2;
  }
  
  public Predicate getTestPredicate()
  {
    return this.m_test;
  }
  
  public Expression getThenExpression()
  {
    return this.m_then;
  }
  
  public Expression getElseExpression()
  {
    return this.m_else;
  }
  
  public void setTestPredicate(Predicate paramPredicate)
  {
    this.m_test.removeExpressionListener(this);
    this.m_test = paramPredicate;
    if (hasListeners()) {
      paramPredicate.addExpressionListener(this);
    }
    fireExpressionChange();
  }
  
  public void setThenExpression(Expression paramExpression)
  {
    this.m_then.removeExpressionListener(this);
    this.m_then = paramExpression;
    if (hasListeners()) {
      paramExpression.addExpressionListener(this);
    }
    fireExpressionChange();
  }
  
  public void setElseExpression(Expression paramExpression)
  {
    this.m_else.removeExpressionListener(this);
    this.m_else = paramExpression;
    if (hasListeners()) {
      paramExpression.addExpressionListener(this);
    }
    fireExpressionChange();
  }
  
  public Class getType(Schema paramSchema)
  {
    Class localClass1 = this.m_then.getType(paramSchema);
    Class localClass2 = this.m_else.getType(paramSchema);
    return TypeLib.getSharedType(localClass1, localClass2);
  }
  
  public Object get(Tuple paramTuple)
  {
    return (this.m_test.getBoolean(paramTuple) ? this.m_then : this.m_else).get(paramTuple);
  }
  
  public boolean getBoolean(Tuple paramTuple)
  {
    return (this.m_test.getBoolean(paramTuple) ? this.m_then : this.m_else).getBoolean(paramTuple);
  }
  
  public double getDouble(Tuple paramTuple)
  {
    return (this.m_test.getBoolean(paramTuple) ? this.m_then : this.m_else).getDouble(paramTuple);
  }
  
  public float getFloat(Tuple paramTuple)
  {
    return (this.m_test.getBoolean(paramTuple) ? this.m_then : this.m_else).getFloat(paramTuple);
  }
  
  public int getInt(Tuple paramTuple)
  {
    return (this.m_test.getBoolean(paramTuple) ? this.m_then : this.m_else).getInt(paramTuple);
  }
  
  public long getLong(Tuple paramTuple)
  {
    return (this.m_test.getBoolean(paramTuple) ? this.m_then : this.m_else).getLong(paramTuple);
  }
  
  public void visit(ExpressionVisitor paramExpressionVisitor)
  {
    paramExpressionVisitor.visitExpression(this);
    paramExpressionVisitor.down();
    this.m_test.visit(paramExpressionVisitor);
    paramExpressionVisitor.up();
    paramExpressionVisitor.down();
    this.m_then.visit(paramExpressionVisitor);
    paramExpressionVisitor.up();
    paramExpressionVisitor.down();
    this.m_else.visit(paramExpressionVisitor);
    paramExpressionVisitor.up();
  }
  
  protected void addChildListeners()
  {
    this.m_test.addExpressionListener(this);
    this.m_then.addExpressionListener(this);
    this.m_else.addExpressionListener(this);
  }
  
  protected void removeChildListeners()
  {
    this.m_test.removeExpressionListener(this);
    this.m_then.removeExpressionListener(this);
    this.m_else.removeExpressionListener(this);
  }
  
  public String toString()
  {
    return "IF " + this.m_test.toString() + " THEN " + this.m_then.toString() + " ELSE " + this.m_else.toString();
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\expression\IfExpression.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */