package prefuse.visual;

import java.util.Iterator;
import prefuse.Visualization;
import prefuse.data.Schema;
import prefuse.data.Table;
import prefuse.data.Tuple;
import prefuse.data.util.Index;
import prefuse.util.collections.IntIterator;
import prefuse.visual.tuple.TableAggregateItem;

public class AggregateTable
  extends VisualTable
{
  protected Table m_aggregated = AGGREGATED_SCHEMA.instantiate();
  protected static final String AGGREGATE = "aggregate";
  protected static final String MEMBER_HASH = "hash";
  protected static final String MEMBER = "member";
  protected static final Schema AGGREGATED_SCHEMA = new Schema();
  
  public AggregateTable(Visualization paramVisualization, String paramString)
  {
    this(paramVisualization, paramString, VisualItem.SCHEMA);
  }
  
  public AggregateTable(Visualization paramVisualization, String paramString, Schema paramSchema)
  {
    super(paramVisualization, paramString, paramSchema, TableAggregateItem.class);
    this.m_aggregated.index("aggregate");
    this.m_aggregated.index("hash");
  }
  
  public int getAggregateSize(int paramInt)
  {
    int i = 0;
    AggregatedIterator localAggregatedIterator = new AggregatedIterator(paramInt);
    while (localAggregatedIterator.hasNext())
    {
      i++;
      localAggregatedIterator.next();
    }
    return i;
  }
  
  public void addToAggregate(int paramInt, VisualItem paramVisualItem)
  {
    validRowCheck(paramInt, true);
    if (!aggregateContains(paramInt, paramVisualItem))
    {
      int i = this.m_aggregated.addRow();
      this.m_aggregated.setInt(i, "aggregate", paramInt);
      this.m_aggregated.setInt(i, "hash", getHashCode(paramVisualItem));
      this.m_aggregated.set(i, "member", paramVisualItem);
      fireTableEvent(paramInt, paramInt, -1, 0);
    }
  }
  
  public void removeFromAggregate(int paramInt, VisualItem paramVisualItem)
  {
    validRowCheck(paramInt, true);
    int i = getAggregatedRow(paramInt, paramVisualItem);
    if (i >= 0)
    {
      this.m_aggregated.removeRow(i);
      fireTableEvent(paramInt, paramInt, -1, 0);
    }
  }
  
  public void removeAllFromAggregate(int paramInt)
  {
    clearAggregateMappings(paramInt, true);
  }
  
  protected void clearAggregateMappings(int paramInt, boolean paramBoolean)
  {
    Index localIndex = this.m_aggregated.index("aggregate");
    int i = 0;
    IntIterator localIntIterator = localIndex.rows(paramInt);
    while (localIntIterator.hasNext())
    {
      int j = localIntIterator.nextInt();
      localIntIterator.remove();
      this.m_aggregated.removeRow(j);
      i = 1;
    }
    if ((paramBoolean) && (i != 0)) {
      fireTableEvent(paramInt, paramInt, -1, 0);
    }
  }
  
  public boolean aggregateContains(int paramInt, VisualItem paramVisualItem)
  {
    return getAggregatedRow(paramInt, paramVisualItem) >= 0;
  }
  
  protected int getAggregatedRow(int paramInt, VisualItem paramVisualItem)
  {
    Index localIndex = this.m_aggregated.index("hash");
    int i = getHashCode(paramVisualItem);
    int j = localIndex.get(i);
    if (j < 0) {
      return -1;
    }
    if (this.m_aggregated.getInt(j, "aggregate") == paramInt) {
      return j;
    }
    IntIterator localIntIterator = localIndex.rows(i);
    while (localIntIterator.hasNext())
    {
      j = localIntIterator.nextInt();
      if (this.m_aggregated.getInt(j, "aggregate") == paramInt) {
        return j;
      }
    }
    return -1;
  }
  
  public Iterator aggregatedTuples(int paramInt)
  {
    return new AggregatedIterator(paramInt);
  }
  
  protected int getHashCode(Tuple paramTuple)
  {
    return paramTuple.hashCode();
  }
  
  protected boolean validRowCheck(int paramInt, boolean paramBoolean)
  {
    if (isValidRow(paramInt)) {
      return true;
    }
    if (paramBoolean) {
      throw new IllegalArgumentException("Invalid row value: " + paramInt);
    }
    return false;
  }
  
  protected void fireTableEvent(int paramInt1, int paramInt2, int paramInt3, int paramInt4)
  {
    if ((paramInt3 == -1) && (paramInt4 == -1)) {
      for (int i = paramInt1; i <= paramInt2; i++) {
        clearAggregateMappings(i, false);
      }
    }
    super.fireTableEvent(paramInt1, paramInt2, paramInt3, paramInt4);
  }
  
  static
  {
    AGGREGATED_SCHEMA.addColumn("aggregate", Integer.TYPE);
    AGGREGATED_SCHEMA.addColumn("hash", Integer.TYPE);
    AGGREGATED_SCHEMA.addColumn("member", Tuple.class);
  }
  
  protected class AggregatedIterator
    implements Iterator
  {
    private IntIterator m_rows;
    private Tuple m_next = null;
    
    public AggregatedIterator(int paramInt)
    {
      Index localIndex = AggregateTable.this.m_aggregated.index("aggregate");
      this.m_rows = localIndex.rows(paramInt);
      advance();
    }
    
    public boolean hasNext()
    {
      return this.m_next != null;
    }
    
    public Object next()
    {
      Tuple localTuple = this.m_next;
      advance();
      return localTuple;
    }
    
    private void advance()
    {
      while (this.m_rows.hasNext())
      {
        int i = this.m_rows.nextInt();
        Tuple localTuple = (Tuple)AggregateTable.this.m_aggregated.get(i, "member");
        if (localTuple.isValid())
        {
          this.m_next = localTuple;
          return;
        }
        AggregateTable.this.m_aggregated.removeRow(i);
      }
      this.m_next = null;
    }
    
    public void remove()
    {
      throw new UnsupportedOperationException();
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\visual\AggregateTable.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */