package prefuse.visual;

import java.awt.BasicStroke;
import java.awt.Font;
import java.awt.geom.Rectangle2D;
import prefuse.Visualization;
import prefuse.data.CascadedTable;
import prefuse.data.Schema;
import prefuse.data.Table;
import prefuse.data.expression.Predicate;
import prefuse.visual.tuple.TableVisualItem;

public class VisualTable
  extends CascadedTable
  implements VisualTupleSet
{
  private Visualization m_vis;
  private String m_group;
  
  public VisualTable(Table paramTable, Visualization paramVisualization, String paramString)
  {
    this(paramTable, paramVisualization, paramString, null, VisualItem.SCHEMA);
  }
  
  public VisualTable(Table paramTable, Visualization paramVisualization, String paramString, Predicate paramPredicate)
  {
    this(paramTable, paramVisualization, paramString, paramPredicate, VisualItem.SCHEMA);
  }
  
  public VisualTable(Table paramTable, Visualization paramVisualization, String paramString, Predicate paramPredicate, Schema paramSchema)
  {
    super(paramTable, paramPredicate, null, TableVisualItem.class);
    init(paramVisualization, paramString, paramSchema);
  }
  
  public VisualTable(Visualization paramVisualization, String paramString)
  {
    super(TableVisualItem.class);
    init(paramVisualization, paramString, VisualItem.SCHEMA);
  }
  
  public VisualTable(Visualization paramVisualization, String paramString, Schema paramSchema)
  {
    super(TableVisualItem.class);
    init(paramVisualization, paramString, paramSchema);
  }
  
  public VisualTable(Visualization paramVisualization, String paramString, Schema paramSchema, Class paramClass)
  {
    super(paramClass);
    init(paramVisualization, paramString, paramSchema);
  }
  
  protected void init(Visualization paramVisualization, String paramString, Schema paramSchema)
  {
    setVisualization(paramVisualization);
    setGroup(paramString);
    addColumns(paramSchema);
    if (canGetBoolean(VisualItem.VISIBLE)) {
      index(VisualItem.VISIBLE);
    }
    if (canGetBoolean(VisualItem.STARTVISIBLE)) {
      index(VisualItem.STARTVISIBLE);
    }
    if (canGetBoolean(VisualItem.VALIDATED)) {
      index(VisualItem.VALIDATED);
    }
  }
  
  protected void fireTableEvent(int paramInt1, int paramInt2, int paramInt3, int paramInt4)
  {
    int i;
    if (paramInt4 == 0)
    {
      if (paramInt3 != VisualItem.IDX_VALIDATED) {
        for (i = paramInt1; i <= paramInt2; i++) {
          setValidated(i, false);
        }
      } else {
        for (i = paramInt1; i <= paramInt2; i++) {
          if (!isValidated(i)) {
            this.m_vis.damageReport(getItem(i), getBounds(i));
          }
        }
      }
    }
    else if ((paramInt4 == -1) && (paramInt3 == -1)) {
      for (i = paramInt1; i <= paramInt2; i++) {
        if ((isVisible(i)) && (isValidated(i)))
        {
          VisualItem localVisualItem = (VisualItem)getTuple(i);
          this.m_vis.damageReport(localVisualItem, getBounds(i));
        }
      }
    }
    super.fireTableEvent(paramInt1, paramInt2, paramInt3, paramInt4);
  }
  
  public Visualization getVisualization()
  {
    return this.m_vis;
  }
  
  public void setVisualization(Visualization paramVisualization)
  {
    this.m_vis = paramVisualization;
  }
  
  public String getGroup()
  {
    return this.m_group;
  }
  
  public void setGroup(String paramString)
  {
    this.m_group = paramString;
  }
  
  public VisualItem getItem(int paramInt)
  {
    return (VisualItem)getTuple(paramInt);
  }
  
  public VisualItem addItem()
  {
    return getItem(addRow());
  }
  
  public boolean isValidated(int paramInt)
  {
    return getBoolean(paramInt, VisualItem.VALIDATED);
  }
  
  public void setValidated(int paramInt, boolean paramBoolean)
  {
    setBoolean(paramInt, VisualItem.VALIDATED, paramBoolean);
  }
  
  public boolean isVisible(int paramInt)
  {
    return getBoolean(paramInt, VisualItem.VISIBLE);
  }
  
  public void setVisible(int paramInt, boolean paramBoolean)
  {
    setBoolean(paramInt, VisualItem.VISIBLE, paramBoolean);
  }
  
  public boolean isStartVisible(int paramInt)
  {
    return getBoolean(paramInt, VisualItem.STARTVISIBLE);
  }
  
  public void setStartVisible(int paramInt, boolean paramBoolean)
  {
    setBoolean(paramInt, VisualItem.STARTVISIBLE, paramBoolean);
  }
  
  public boolean isEndVisible(int paramInt)
  {
    return getBoolean(paramInt, VisualItem.ENDVISIBLE);
  }
  
  public void setEndVisible(int paramInt, boolean paramBoolean)
  {
    setBoolean(paramInt, VisualItem.ENDVISIBLE, paramBoolean);
  }
  
  public boolean isInteractive(int paramInt)
  {
    return getBoolean(paramInt, VisualItem.INTERACTIVE);
  }
  
  public void setInteractive(int paramInt, boolean paramBoolean)
  {
    setBoolean(paramInt, VisualItem.INTERACTIVE, paramBoolean);
  }
  
  public boolean isExpanded(int paramInt)
  {
    return getBoolean(paramInt, VisualItem.EXPANDED);
  }
  
  public void setExpanded(int paramInt, boolean paramBoolean)
  {
    setBoolean(paramInt, VisualItem.EXPANDED, paramBoolean);
  }
  
  public boolean isFixed(int paramInt)
  {
    return getBoolean(paramInt, VisualItem.FIXED);
  }
  
  public void setFixed(int paramInt, boolean paramBoolean)
  {
    setBoolean(paramInt, VisualItem.FIXED, paramBoolean);
  }
  
  public boolean isHighlighted(int paramInt)
  {
    return getBoolean(paramInt, VisualItem.HIGHLIGHT);
  }
  
  public void setHighlighted(int paramInt, boolean paramBoolean)
  {
    setBoolean(paramInt, VisualItem.HIGHLIGHT, paramBoolean);
  }
  
  public boolean isHover(int paramInt)
  {
    return getBoolean(paramInt, VisualItem.HOVER);
  }
  
  public void setHover(int paramInt, boolean paramBoolean)
  {
    setBoolean(paramInt, VisualItem.HOVER, paramBoolean);
  }
  
  public double getX(int paramInt)
  {
    return getDouble(paramInt, VisualItem.X);
  }
  
  public void setX(int paramInt, double paramDouble)
  {
    setDouble(paramInt, VisualItem.X, paramDouble);
  }
  
  public double getY(int paramInt)
  {
    return getDouble(paramInt, VisualItem.Y);
  }
  
  public void setY(int paramInt, double paramDouble)
  {
    setDouble(paramInt, VisualItem.Y, paramDouble);
  }
  
  public double getStartX(int paramInt)
  {
    return getDouble(paramInt, VisualItem.STARTX);
  }
  
  public void setStartX(int paramInt, double paramDouble)
  {
    setDouble(paramInt, VisualItem.STARTX, paramDouble);
  }
  
  public double getStartY(int paramInt)
  {
    return getDouble(paramInt, VisualItem.STARTY);
  }
  
  public void setStartY(int paramInt, double paramDouble)
  {
    setDouble(paramInt, VisualItem.STARTY, paramDouble);
  }
  
  public double getEndX(int paramInt)
  {
    return getDouble(paramInt, VisualItem.ENDX);
  }
  
  public void setEndX(int paramInt, double paramDouble)
  {
    setDouble(paramInt, VisualItem.ENDX, paramDouble);
  }
  
  public double getEndY(int paramInt)
  {
    return getDouble(paramInt, VisualItem.ENDY);
  }
  
  public void setEndY(int paramInt, double paramDouble)
  {
    setDouble(paramInt, VisualItem.ENDY, paramDouble);
  }
  
  public Rectangle2D getBounds(int paramInt)
  {
    return (Rectangle2D)get(paramInt, VisualItem.BOUNDS);
  }
  
  public void setBounds(int paramInt, double paramDouble1, double paramDouble2, double paramDouble3, double paramDouble4)
  {
    getBounds(paramInt).setRect(paramDouble1, paramDouble2, paramDouble3, paramDouble4);
    fireTableEvent(paramInt, paramInt, getColumnNumber(VisualItem.BOUNDS), 0);
  }
  
  public int getStrokeColor(int paramInt)
  {
    return getInt(paramInt, VisualItem.STROKECOLOR);
  }
  
  public void setStrokeColor(int paramInt1, int paramInt2)
  {
    setInt(paramInt1, VisualItem.STROKECOLOR, paramInt2);
  }
  
  public int getStartStrokeColor(int paramInt)
  {
    return getInt(paramInt, VisualItem.STARTSTROKECOLOR);
  }
  
  public void setStartStrokeColor(int paramInt1, int paramInt2)
  {
    setInt(paramInt1, VisualItem.STARTSTROKECOLOR, paramInt2);
  }
  
  public int getEndStrokeColor(int paramInt)
  {
    return getInt(paramInt, VisualItem.ENDSTROKECOLOR);
  }
  
  public void setEndStrokeColor(int paramInt1, int paramInt2)
  {
    setInt(paramInt1, VisualItem.ENDSTROKECOLOR, paramInt2);
  }
  
  public int getFillColor(int paramInt)
  {
    return getInt(paramInt, VisualItem.FILLCOLOR);
  }
  
  public void setFillColor(int paramInt1, int paramInt2)
  {
    setInt(paramInt1, VisualItem.FILLCOLOR, paramInt2);
  }
  
  public int getStartFillColor(int paramInt)
  {
    return getInt(paramInt, VisualItem.STARTFILLCOLOR);
  }
  
  public void setStartFillColor(int paramInt1, int paramInt2)
  {
    setInt(paramInt1, VisualItem.STARTFILLCOLOR, paramInt2);
  }
  
  public int getEndFillColor(int paramInt)
  {
    return getInt(paramInt, VisualItem.ENDFILLCOLOR);
  }
  
  public void setEndFillColor(int paramInt1, int paramInt2)
  {
    setInt(paramInt1, VisualItem.ENDFILLCOLOR, paramInt2);
  }
  
  public int getTextColor(int paramInt)
  {
    return getInt(paramInt, VisualItem.TEXTCOLOR);
  }
  
  public void setTextColor(int paramInt1, int paramInt2)
  {
    setInt(paramInt1, VisualItem.TEXTCOLOR, paramInt2);
  }
  
  public int getStartTextColor(int paramInt)
  {
    return getInt(paramInt, VisualItem.STARTTEXTCOLOR);
  }
  
  public void setStartTextColor(int paramInt1, int paramInt2)
  {
    setInt(paramInt1, VisualItem.STARTTEXTCOLOR, paramInt2);
  }
  
  public int getEndTextColor(int paramInt)
  {
    return getInt(paramInt, VisualItem.ENDTEXTCOLOR);
  }
  
  public void setEndTextColor(int paramInt1, int paramInt2)
  {
    setInt(paramInt1, VisualItem.ENDTEXTCOLOR, paramInt2);
  }
  
  public double getSize(int paramInt)
  {
    return getDouble(paramInt, VisualItem.SIZE);
  }
  
  public void setSize(int paramInt, double paramDouble)
  {
    setDouble(paramInt, VisualItem.SIZE, paramDouble);
  }
  
  public double getStartSize(int paramInt)
  {
    return getDouble(paramInt, VisualItem.STARTSIZE);
  }
  
  public void setStartSize(int paramInt, double paramDouble)
  {
    setDouble(paramInt, VisualItem.STARTSIZE, paramDouble);
  }
  
  public double getEndSize(int paramInt)
  {
    return getDouble(paramInt, VisualItem.ENDSIZE);
  }
  
  public void setEndSize(int paramInt, double paramDouble)
  {
    setDouble(paramInt, VisualItem.ENDSIZE, paramDouble);
  }
  
  public int getShape(int paramInt)
  {
    return getInt(paramInt, VisualItem.SHAPE);
  }
  
  public void setShape(int paramInt1, int paramInt2)
  {
    setInt(paramInt1, VisualItem.SHAPE, paramInt2);
  }
  
  public BasicStroke getStroke(int paramInt)
  {
    return (BasicStroke)get(paramInt, VisualItem.STROKE);
  }
  
  public void setStroke(int paramInt, BasicStroke paramBasicStroke)
  {
    set(paramInt, VisualItem.STROKE, paramBasicStroke);
  }
  
  public Font getFont(int paramInt)
  {
    return (Font)get(paramInt, VisualItem.FONT);
  }
  
  public void setFont(int paramInt, Font paramFont)
  {
    set(paramInt, VisualItem.FONT, paramFont);
  }
  
  public Font getStartFont(int paramInt)
  {
    return (Font)get(paramInt, VisualItem.STARTFONT);
  }
  
  public void setStartFont(int paramInt, Font paramFont)
  {
    set(paramInt, VisualItem.STARTFONT, paramFont);
  }
  
  public Font getEndFont(int paramInt)
  {
    return (Font)get(paramInt, VisualItem.ENDFONT);
  }
  
  public void setEndFont(int paramInt, Font paramFont)
  {
    set(paramInt, VisualItem.ENDFONT, paramFont);
  }
  
  public double getDOI(int paramInt)
  {
    return getDouble(paramInt, VisualItem.DOI);
  }
  
  public void setDOI(int paramInt, double paramDouble)
  {
    setDouble(paramInt, VisualItem.DOI, paramDouble);
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\visual\VisualTable.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */